--[[ NavHelper Lite by MrInfernal  v2.5

Improve usage of MQ2Nav and streamline travel
Allows the user to enter a port spell or trigger an evac
]]

---@type Mq
local mq = require('mq')

-- Import the zone and port files
DruPorts = require('druports')
WizPorts = require('wizports')

AllPorts = {}
for _, addwiz in ipairs(WizPorts) do
    table.insert(AllPorts, addwiz)
end
for _, adddru in ipairs(DruPorts) do
    table.insert(AllPorts, adddru)
end

-- Set default values
local stayOpen = true
local useCWTN = false       -- Disable if not using CWTN Druid

-- Create a table of the porter character name and class checks
local function setMyPorters(porters)
    local myPorters = {}

    if mq.TLO.Group() then
        for char in porters:gmatch("[^|]+") do
            if mq.TLO.Group.Member(char).Class() == 'Druid' or mq.TLO.Group.Member(char).Class() == 'Wizard' then
                local name = mq.TLO.Group.Member(char).Name()
                local class = mq.TLO.Group.Member(char).Class()
                local level = mq.TLO.Group.Member(char).Level()
                table.insert(myPorters, {name = name, class = class, level = level})
            end
        end
    else
        for char2 in porters:gmatch("[^|]+") do
            if mq.TLO.Me.Class.Name() == 'Druid' or mq.TLO.Me.Class.Name() == 'Wizard' then
                local name = mq.TLO.Me.Name()
                local class = mq.TLO.Me.Class()
                local level = mq.TLO.Me.Level()
                table.insert(myPorters, {name = name, class = class, level = level})
            end
        end
    end

    return myPorters
end

-- Use the myPorters table to check for port classes
local myPorters = setMyPorters(tostring(mq.TLO.DanNet.Peers))

local hasDruid = false
local hasWizard = false

for i, porter in ipairs(myPorters) do
    if porter.class == "Druid" then
        hasDruid = true
    elseif porter.class == "Wizard" then
        hasWizard = true
    end
end

local function checkCWTN()
    if mq.TLO.Plugin('mq2druid').IsLoaded() and hasDruid then
        useCWTN = true
    else
        useCWTN = false
    end
end

local function portCheck(input)
    local results = {}

    for _, port in ipairs(AllPorts) do
        if port.loc == input then
            table.insert(results, port)
        end
    end

    return results
end

local function doEvac()
    local evacSpells = {'Exodus', 'Succor', 'Lesser Succor', 'Evacuate', 'Lesser Evacuate'}
    local portName = ''
    local spellGem = 5
    local myPorters = setMyPorters(tostring(mq.TLO.DanNet.Peers))
    local porter = ''

    for _, char in ipairs(myPorters) do
        for _, port in ipairs(evacSpells) do
            if char.class == 'Druid' and char.level >= 57 and port == 'Succor' then
                portName = port
                porter = char.name
                if useCWTN then
                    mq.cmdf('/squelch /dex %s /dru pause on', porter)
                end   
                break
            elseif char.class == 'Druid' and char.level >= 18 and port == 'Lesser Succor' then
                portName = port
                porter = char.name
                if useCWTN then
                    mq.cmdf('/squelch /dex %s /dru pause on', porter)
                end
                break
            elseif char.class == 'Wizard' and char.level >= 57 and port == 'Evacuate' then
                portName = port
                porter = char.name
                break
            elseif char.class == 'Wizard' and char.level >= 18 and port == 'Lesser Evacuate' then
                portName = port
                porter = char.name
                break
            end
        end
    end

    if useCWTN then
        if mq.TLO.Me.Class.Name() == 'Druid' then
            mq.cmd('/squelch /dru pause on')
        end
    end
    
    if mq.TLO.Me.Class == 'Druid' or mq.TLO.Me.Class == 'Wizard' then
        -- if Exodus not ready
        mq.cmdf('/squelch /if (!${Cast.Ready[exodus]}) /casting "%s" %s', portName, spellGem)
        mq.cmdf('/squelch /if (!${Cast.Ready[exodus]}) /dgt all Casting "%s"!', portName)
        -- if Exodus is ready
        mq.cmdf('/squelch /if (${Cast.Ready[exodus]}) /alt act 43')
        mq.cmdf('/squelch /if (${Cast.Ready[exodus]}) /dgt all Using Exodus AA!')
    else
        -- use exodus if ready
        mq.cmdf('/timed 1 /noparse /squelch /dex %s /if (${Cast.Ready[exodus]}) /alt act 43', porter)
        mq.cmdf('/timed 2 /noparse /squelch /dex %s /if (${Cast.Ready[exodus]}) /dgt all Using Exodus AA!', porter)
        -- Have the porter cast the spell. Will report if need to memorize or don't have the spell
        mq.cmdf('/timed 3 /noparse /squelch /dex %s /if (!${Cast.Ready[exodus]}) /if (${Me.Book["%s"]}>0) /casting "%s" %s', porter, portName, portName, spellGem)
        mq.cmdf('/timed 4 /noparse /squelch /dex %s /if (!${Cast.Ready[exodus]}) /if (!${Me.Book["%s"]}>0) /dgt all Spell not known!', porter, portName)

        -- Have to memorizea
        mq.cmdf('/timed 5 /noparse /squelch /dex %s /if (!${Cast.Ready[exodus]}) /if (${Cast.Status.Equal[CM]}) /timed 5 /dgt all Memorizing "%s"! Ready in 10 seconds', porter, portName)
        mq.cmdf('/timed 140 /noparse /squelch /dex %s /if (!${Cast.Ready[exodus]}) /if (${Cast.Timing}>2000) /dgt all Casting -> %s <- in ${Math.Calc[${Cast.Timing}/1000]} seconds!', porter, portName)

        -- Already memorized
        mq.cmdf('/timed 5 /noparse /squelch /dex %s /if (!${Cast.Ready[exodus]}) /if (${Cast.Timing}>2000) /dgt all Casting -> %s <- in ${Math.Calc[${Cast.Timing}/1000]} seconds!', porter, portName)
        mq.cmdf('/timed 80 /noparse /squelch /dex %s /if (!${Cast.Ready[exodus]}) /if (${Cast.Timing}>5000) /dgt all ${Math.Calc[${Cast.Timing}/1000]} seconds remaining!', porter, portName)

        -- Out of mana
        mq.cmdf('/timed 5 /noparse /squelch /dex %s /if (!${Cast.Ready[exodus]}) /if (${Cast.Status.Equal[CAST_OUTOFMANA]}) /dgt \arall Out of mana!', porter)
        --mq.cmdf('/timed 8 /noparse /squelch /dex %s /if (${Cast.Status.Equal[CAST_OUTOFMANA]}) /if (!${Me.Sitting}) /sit', porter)
    end
end

-- Create a function to handle the teleporting
local function boundPort(input, gem)
    local found = false
    local portName = ''
    local spellGem = 5
    local myPorter = setMyPorters(tostring(mq.TLO.DanNet.Peers))
    local porter = ''

    for _, port in ipairs(AllPorts) do
        if string.lower(port.name) == string.lower(input) then
            found = true
            portName = port.name
            if gem ~= nil then
                spellGem = gem
            end
            break
        end
    end

    if found then
        if string.find(portName, 'Ring') or string.find(portName, 'Circle') or string.find(portName, 'Wind of the') then
            for _, char in ipairs(myPorter) do
                if char.class == 'Druid' then
                    porter = char.name
                    if useCWTN then
                        mq.cmdf('/squelch /dex %s /dru pause on', porter)
                    end
                end
            end
        elseif string.find(portName, 'Gate') or string.find(portName, 'Portal')  or string.find(portName, 'Relocation')then
            for _, char in ipairs(myPorter) do
                if char.class == 'Wizard' then
                    porter = char.name
                end
            end
        end

        if useCWTN then
            if mq.TLO.Me.Class.Name() == 'Druid' then
                mq.cmd('/squelch /dru pause on')
            end
        end
        
        if mq.TLO.Me.Class() == 'Druid' or mq.TLO.Me.Class() == 'Wizard' then
            mq.cmdf('/squelch /casting "%s" %d', portName, spellGem)
            mq.cmdf('/dgt all Casting "%s"!', portName)
        else
            -- Have the porter cast the spell. Will report if need to memorize or don't have the spell
            mq.cmdf('/noparse /squelch /dex %s /if (${Me.Book["%s"]}>0) /casting "%s" %d', porter, portName, portName, spellGem)
            mq.cmdf('/timed 2 /noparse /squelch /dex %s /if (!${Me.Book["%s"]}>0) /dgt all Spell not known!', porter, portName)

            -- Have to memorize
            mq.cmdf('/timed 5 /noparse /squelch /dex %s /if (${Cast.Status.Equal[CM]}) /timed 5 /dgt all Memorizing "%s"! Ready in 10 seconds', porter, portName)
            mq.cmdf('/timed 140 /noparse /squelch /dex %s /if (${Cast.Timing}>2000) /dgt all Casting -> %s <- in ${Math.Calc[${Cast.Timing}/1000]} seconds!', porter, portName)

            -- Already memorized
            mq.cmdf('/timed 5 /noparse /squelch /dex %s /if (${Cast.Timing}>2000) /dgt all Casting -> %s <- in ${Math.Calc[${Cast.Timing}/1000]} seconds!', porter, portName)
            mq.cmdf('/timed 80 /noparse /squelch /dex %s /if (${Cast.Timing}>5000) /dgt all ${Math.Calc[${Cast.Timing}/1000]} seconds remaining!', porter, portName)

            -- Out of mana
            mq.cmdf('/timed 5 /noparse /squelch /dex %s /if (${Cast.Status.Equal[CAST_OUTOFMANA]}) /dgt all Out of mana!', porter)
            mq.cmdf('/timed 8 /noparse /squelch /dex %s /if (${Cast.Status.Equal[CAST_OUTOFMANA]}) /if (!${Me.Sitting}) /sit', porter)
        end
    else
        print('Spell not found!')
    end
end

-- Commands to run after zoning
local function justZoned()
    if useCWTN then
        if mq.TLO.Me.Class() == 'Druid' then
            mq.cmd('/squelch /dru pause off')
        else
            for _, char in ipairs(myPorters) do
                if char.class == 'Druid' then
                    mq.cmdf('/squelch /dex %s /dru pause off', char.name)
                --[[elseif char.class == 'Wizard' then
                    mq.cmdf('/squelch /dex %s /rgstart', char.name)
                    mq.cmdf('/squelch /dex %s /dgt all Restarting RGMercs macro!', char.name)]]
                end
            end
        end
    end
end

local function exitProg()
    mq.unbind('/nh')
    mq.unbind('/nhevac')
    mq.unbind('/nhexit')

    stayOpen = false
end

local function showHelp()
    print('\aoNavHelper Lite \axby \arMrInfernal')
    print('\ao/nh \ag"port spell" \at# \ax\ax\ax-- Use the port spell. (Optional) Add a spell gem number')
    print('Ensure the port spell is in quotes ("). Ex: /nh "circle of knowledge" 3')
    print('\ao/nhevac \ax-- Have yourself or group member use an evac')
    print('\ao/nhexit \ax-- Exit the program')
end
-------------- End of functions

-- Creates slash commands for porting. Allows creation of hotkeys
-- ex: /nh "circle of knowledge"
mq.bind("/nh", boundPort)
mq.bind('/nhevac', doEvac)
mq.bind('/nhexit', exitProg)

-- Set up event listeners
mq.event('zoned', 'You have entered*#*', justZoned)

-- See if MQ2Druid is loaded and a grouped druid. If so, then toggle the useCWTN box in settings
checkCWTN()
portCheck()
showHelp()

while stayOpen do
    mq.doevents()
    mq.delay(1000) -- equivalent to '1s'
end